import { crayon } from "https://deno.land/x/crayon@3.3.3/mod.ts";
import { z } from "https://deno.land/x/zod@v3.22.4/mod.ts";

export class ApiClient {
	url;
	token;
	constructor(hostname: string, token: string) {
		this.url = `https://${hostname}`;
		this.token = token;
	}

	async fetch(path: string) {
		console.log(crayon.lightBlack("\tfetching", crayon.bold(path)));
		const headers = new Headers([["PRIVATE-TOKEN", this.token]]);
		const result = await fetch(this.url + "/" + path, { headers });
		const parsed = await result.json();
		return parsed as unknown;
	}

	async fetch_parse<T>(path: string, parser: z.ZodType<T>) {
		const unkn = await this.fetch(path);
		return parser.parse(unkn);
	}

	async fetch_parse_paginated<T>(path: string, parser: z.ZodType<T>) {
		const result = [] as T[];
		let page = 1;
		while (true) {
			const fetched = await this.fetch_parse(path + "?per_page=100&page=" + page, parser.array());
			result.push(...fetched);
			if (fetch.length < 100) break;
			page += 1;
		}
		return result;
	}
}
